@@ -7,6 +7,8 @@ import (
 	"os"
 	"path/filepath"
 
+	"github.com/antonmedv/expr"
+
 	"github.com/pkg/errors"
 
 	"github.com/crowdsecurity/crowdsec/pkg/csconfig"
@@ -44,7 +46,7 @@ func backupConfigToDirectory(dirPath string) error {
 		return errors.Wrapf(err, "while checking parent directory %s existence", parentDir)
 	}
 
-	if err = os.Mkdir(dirPath, 0600); err != nil {
+	if err = os.Mkdir(dirPath, 0700); err != nil {
 		return fmt.Errorf("error while creating %s : %s", dirPath, err)
 	}
 
@@ -68,7 +70,7 @@ func backupConfigToDirectory(dirPath string) error {
 	}
 
 	acquisBackupDir := dirPath + "/acquis/"
-	if err = os.Mkdir(acquisBackupDir, 0600); err != nil {
+	if err = os.Mkdir(acquisBackupDir, 0700); err != nil {
 		return fmt.Errorf("error while creating %s : %s", acquisBackupDir, err)
 	}
 
@@ -215,7 +217,7 @@ func restoreConfigFromDirectory(dirPath string) error {
 
 	/*if there is a acquisition dir, restore its content*/
 	if csConfig.Crowdsec.AcquisitionDirPath != "" {
-		if err = os.Mkdir(csConfig.Crowdsec.AcquisitionDirPath, 0600); err != nil {
+		if err = os.Mkdir(csConfig.Crowdsec.AcquisitionDirPath, 0700); err != nil {
 			return fmt.Errorf("error while creating %s : %s", csConfig.Crowdsec.AcquisitionDirPath, err)
 		}
 
@@ -274,72 +276,125 @@ func restoreConfigFromDirectory(dirPath string) error {
 func NewConfigCmd() *cobra.Command {
 
 	var cmdConfig = &cobra.Command{
-		Use:   "config [command]",
-		Short: "Allows to view current config",
-		Args:  cobra.ExactArgs(0),
+		Use:               "config [command]",
+		Short:             "Allows to view current config",
+		Args:              cobra.ExactArgs(0),
+		DisableAutoGenTag: true,
+	}
+	var key string
+	type Env struct {
+		Config *csconfig.Config
 	}
 	var cmdConfigShow = &cobra.Command{
-		Use:   "show",
-		Short: "Displays current config",
-		Long:  `Displays the current cli configuration.`,
-		Args:  cobra.ExactArgs(0),
+		Use:               "show",
+		Short:             "Displays current config",
+		Long:              `Displays the current cli configuration.`,
+		Args:              cobra.ExactArgs(0),
+		DisableAutoGenTag: true,
 		Run: func(cmd *cobra.Command, args []string) {
+
+			if key != "" {
+				program, err := expr.Compile(key, expr.Env(Env{}))
+				if err != nil {
+					log.Fatal(err)
+				}
+				output, err := expr.Run(program, Env{Config: csConfig})
+				if err != nil {
+					log.Fatal(err)
+				}
+				switch csConfig.Cscli.Output {
+				case "human", "raw":
+					switch output.(type) {
+					case string:
+						fmt.Printf("%s\n", output)
+					case int:
+						fmt.Printf("%d\n", output)
+					default:
+						fmt.Printf("%v\n", output)
+					}
+				case "json":
+					data, err := json.MarshalIndent(output, "", "  ")
+					if err != nil {
+						log.Fatalf("failed to marshal configuration: %s", err)
+					}
+					fmt.Printf("%s\n", string(data))
+				}
+				return
+			}
+
 			switch csConfig.Cscli.Output {
 			case "human":
 				fmt.Printf("Global:\n")
-				fmt.Printf("   - Configuration Folder   : %s\n", csConfig.ConfigPaths.ConfigDir)
-				fmt.Printf("   - Data Folder            : %s\n", csConfig.ConfigPaths.DataDir)
-				fmt.Printf("   - Log Folder             : %s\n", csConfig.Common.LogDir)
-				fmt.Printf("   - Hub Folder             : %s\n", csConfig.ConfigPaths.HubDir)
-				fmt.Printf("   - Simulation File        : %s\n", csConfig.ConfigPaths.SimulationFilePath)
-				fmt.Printf("   - Log level              : %s\n", csConfig.Common.LogLevel)
-				fmt.Printf("   - Log Media              : %s\n", csConfig.Common.LogMedia)
-				fmt.Printf("Crowdsec:\n")
-				fmt.Printf("  - Acquisition File        : %s\n", csConfig.Crowdsec.AcquisitionFilePath)
-				fmt.Printf("  - Parsers routines        : %d\n", csConfig.Crowdsec.ParserRoutinesCount)
-				fmt.Printf("cscli:\n")
-				fmt.Printf("  - Output                  : %s\n", csConfig.Cscli.Output)
-				fmt.Printf("  - Hub Branch              : %s\n", csConfig.Cscli.HubBranch)
-				fmt.Printf("  - Hub Folder              : %s\n", csConfig.Cscli.HubDir)
-				fmt.Printf("API Client:\n")
-				fmt.Printf("  - URL                     : %s\n", csConfig.API.Client.Credentials.URL)
-				fmt.Printf("  - Login                   : %s\n", csConfig.API.Client.Credentials.Login)
-				fmt.Printf("  - Credentials File        : %s\n", csConfig.API.Client.CredentialsFilePath)
-				fmt.Printf("Local API Server:\n")
-				fmt.Printf("  - Listen URL              : %s\n", csConfig.API.Server.ListenURI)
-				fmt.Printf("  - Profile File            : %s\n", csConfig.API.Server.ProfilesPath)
-				if csConfig.API.Server.TLS != nil {
-					if csConfig.API.Server.TLS.CertFilePath != "" {
-						fmt.Printf("  - Cert File : %s\n", csConfig.API.Server.TLS.CertFilePath)
+				if csConfig.ConfigPaths != nil {
+					fmt.Printf("   - Configuration Folder   : %s\n", csConfig.ConfigPaths.ConfigDir)
+					fmt.Printf("   - Data Folder            : %s\n", csConfig.ConfigPaths.DataDir)
+					fmt.Printf("   - Hub Folder             : %s\n", csConfig.ConfigPaths.HubDir)
+					fmt.Printf("   - Simulation File        : %s\n", csConfig.ConfigPaths.SimulationFilePath)
+				}
+				if csConfig.Common != nil {
+					fmt.Printf("   - Log Folder             : %s\n", csConfig.Common.LogDir)
+					fmt.Printf("   - Log level              : %s\n", csConfig.Common.LogLevel)
+					fmt.Printf("   - Log Media              : %s\n", csConfig.Common.LogMedia)
+				}
+				if csConfig.Crowdsec != nil {
+					fmt.Printf("Crowdsec:\n")
+					fmt.Printf("  - Acquisition File        : %s\n", csConfig.Crowdsec.AcquisitionFilePath)
+					fmt.Printf("  - Parsers routines        : %d\n", csConfig.Crowdsec.ParserRoutinesCount)
+				}
+				if csConfig.Cscli != nil {
+					fmt.Printf("cscli:\n")
+					fmt.Printf("  - Output                  : %s\n", csConfig.Cscli.Output)
+					fmt.Printf("  - Hub Branch              : %s\n", csConfig.Cscli.HubBranch)
+					fmt.Printf("  - Hub Folder              : %s\n", csConfig.Cscli.HubDir)
+				}
+				if csConfig.API != nil {
+					if csConfig.API.Client != nil && csConfig.API.Client.Credentials != nil {
+						fmt.Printf("API Client:\n")
+						fmt.Printf("  - URL                     : %s\n", csConfig.API.Client.Credentials.URL)
+						fmt.Printf("  - Login                   : %s\n", csConfig.API.Client.Credentials.Login)
+						fmt.Printf("  - Credentials File        : %s\n", csConfig.API.Client.CredentialsFilePath)
 					}
-					if csConfig.API.Server.TLS.KeyFilePath != "" {
-						fmt.Printf("  - Key File  : %s\n", csConfig.API.Server.TLS.KeyFilePath)
+					if csConfig.API.Server != nil {
+						fmt.Printf("Local API Server:\n")
+						fmt.Printf("  - Listen URL              : %s\n", csConfig.API.Server.ListenURI)
+						fmt.Printf("  - Profile File            : %s\n", csConfig.API.Server.ProfilesPath)
+						if csConfig.API.Server.TLS != nil {
+							if csConfig.API.Server.TLS.CertFilePath != "" {
+								fmt.Printf("  - Cert File : %s\n", csConfig.API.Server.TLS.CertFilePath)
+							}
+							if csConfig.API.Server.TLS.KeyFilePath != "" {
+								fmt.Printf("  - Key File  : %s\n", csConfig.API.Server.TLS.KeyFilePath)
+							}
+						}
+						if csConfig.API.Server.OnlineClient != nil && csConfig.API.Server.OnlineClient.Credentials != nil {
+							fmt.Printf("Central API:\n")
+							fmt.Printf("  - URL                     : %s\n", csConfig.API.Server.OnlineClient.Credentials.URL)
+							fmt.Printf("  - Login                   : %s\n", csConfig.API.Server.OnlineClient.Credentials.Login)
+							fmt.Printf("  - Credentials File        : %s\n", csConfig.API.Server.OnlineClient.CredentialsFilePath)
+						}
 					}
 				}
-				fmt.Printf("  - Database:\n")
-				fmt.Printf("      - Type                : %s\n", csConfig.DbConfig.Type)
-				switch csConfig.DbConfig.Type {
-				case "sqlite":
-					fmt.Printf("      - Path                : %s\n", csConfig.DbConfig.DbPath)
-				case "mysql", "postgresql", "postgres":
-					fmt.Printf("      - Host                : %s\n", csConfig.DbConfig.Host)
-					fmt.Printf("      - Port                : %d\n", csConfig.DbConfig.Port)
-					fmt.Printf("      - User                : %s\n", csConfig.DbConfig.User)
-					fmt.Printf("      - DB Name             : %s\n", csConfig.DbConfig.DbName)
-				}
-				if csConfig.DbConfig.Flush != nil {
-					if *csConfig.DbConfig.Flush.MaxAge != "" {
-						fmt.Printf("      - Flush age           : %s\n", *csConfig.DbConfig.Flush.MaxAge)
+				if csConfig.DbConfig != nil {
+					fmt.Printf("  - Database:\n")
+					fmt.Printf("      - Type                : %s\n", csConfig.DbConfig.Type)
+					switch csConfig.DbConfig.Type {
+					case "sqlite":
+						fmt.Printf("      - Path                : %s\n", csConfig.DbConfig.DbPath)
+					case "mysql", "postgresql", "postgres":
+						fmt.Printf("      - Host                : %s\n", csConfig.DbConfig.Host)
+						fmt.Printf("      - Port                : %d\n", csConfig.DbConfig.Port)
+						fmt.Printf("      - User                : %s\n", csConfig.DbConfig.User)
+						fmt.Printf("      - DB Name             : %s\n", csConfig.DbConfig.DbName)
 					}
-					if *csConfig.DbConfig.Flush.MaxItems != 0 {
-						fmt.Printf("      - Flush size          : %d\n", *csConfig.DbConfig.Flush.MaxItems)
+					if csConfig.DbConfig.Flush != nil {
+						if *csConfig.DbConfig.Flush.MaxAge != "" {
+							fmt.Printf("      - Flush age           : %s\n", *csConfig.DbConfig.Flush.MaxAge)
+						}
+						if *csConfig.DbConfig.Flush.MaxItems != 0 {
+							fmt.Printf("      - Flush size          : %d\n", *csConfig.DbConfig.Flush.MaxItems)
+						}
 					}
 				}
-
-				fmt.Printf("Central API:\n")
-				fmt.Printf("  - URL                     : %s\n", csConfig.API.Server.OnlineClient.Credentials.URL)
-				fmt.Printf("  - Login                   : %s\n", csConfig.API.Server.OnlineClient.Credentials.Login)
-				fmt.Printf("  - Credentials File        : %s\n", csConfig.API.Server.OnlineClient.CredentialsFilePath)
 			case "json":
 				data, err := json.MarshalIndent(csConfig, "", "  ")
 				if err != nil {
@@ -355,10 +410,11 @@ func NewConfigCmd() *cobra.Command {
 			}
 		},
 	}
+	cmdConfigShow.Flags().StringVar(&key, "key", "", "Display only this value (config.API.Server.ListenURI")
 	cmdConfig.AddCommand(cmdConfigShow)
 
 	var cmdConfigBackup = &cobra.Command{
-		Use:   "backup <directory>",
+		Use:   `backup "directory"`,
 		Short: "Backup current config",
 		Long: `Backup the current crowdsec configuration including :
 
@@ -368,11 +424,15 @@ func NewConfigCmd() *cobra.Command {
 - List of scenarios, parsers, postoverflows and collections that are up-to-date
 - Tainted/local/out-of-date scenarios, parsers, postoverflows and collections
 - Backup of API credentials (local API and online API)`,
-		Example: `cscli config backup ./my-backup`,
-		Args:    cobra.ExactArgs(1),
+		Example:           `cscli config backup ./my-backup`,
+		Args:              cobra.ExactArgs(1),
+		DisableAutoGenTag: true,
 		Run: func(cmd *cobra.Command, args []string) {
 			var err error
-			if err = cwhub.GetHubIdx(csConfig.Cscli); err != nil {
+			if err := csConfig.LoadHub(); err != nil {
+				log.Fatalf(err.Error())
+			}
+			if err = cwhub.GetHubIdx(csConfig.Hub); err != nil {
 				log.Fatalf("Failed to get Hub index : %v", err)
 				log.Infoln("Run 'sudo cscli hub update' to get the hub index")
 			}
@@ -384,9 +444,9 @@ func NewConfigCmd() *cobra.Command {
 	cmdConfig.AddCommand(cmdConfigBackup)
 
 	var cmdConfigRestore = &cobra.Command{
-		Use:   "restore <directory>",
-		Short: "Restore config in backup <directory>",
-		Long: `Restore the crowdsec configuration from specified backup <directory> including:
+		Use:   `restore "directory"`,
+		Short: `Restore config in backup "directory"`,
+		Long: `Restore the crowdsec configuration from specified backup "directory" including:
 
 - Main config (config.yaml)
 - Simulation config (simulation.yaml)
@@ -394,10 +454,14 @@ func NewConfigCmd() *cobra.Command {
 - List of scenarios, parsers, postoverflows and collections that are up-to-date
 - Tainted/local/out-of-date scenarios, parsers, postoverflows and collections
 - Backup of API credentials (local API and online API)`,
-		Args: cobra.ExactArgs(1),
+		Args:              cobra.ExactArgs(1),
+		DisableAutoGenTag: true,
 		Run: func(cmd *cobra.Command, args []string) {
 			var err error
-			if err = cwhub.GetHubIdx(csConfig.Cscli); err != nil {
+			if err := csConfig.LoadHub(); err != nil {
+				log.Fatalf(err.Error())
+			}
+			if err = cwhub.GetHubIdx(csConfig.Hub); err != nil {
 				log.Fatalf("Failed to get Hub index : %v", err)
 				log.Infoln("Run 'sudo cscli hub update' to get the hub index")
 			}
