@@ -48,6 +48,7 @@ type Leaky struct {
 	Mapkey string
 	// chan for signaling
 	Signal       chan bool `json:"-"`
+	Suicide      chan bool `json:"-"`
 	Reprocess    bool
 	Simulated    bool
 	Uuid         string
@@ -77,7 +78,7 @@ var BucketsPour = prometheus.NewCounterVec(
 		Name: "cs_bucket_poured_total",
 		Help: "Total events were poured in bucket.",
 	},
-	[]string{"source", "name"},
+	[]string{"source", "type", "name"},
 )
 
 var BucketsOverflow = prometheus.NewCounterVec(
@@ -88,6 +89,14 @@ var BucketsOverflow = prometheus.NewCounterVec(
 	[]string{"name"},
 )
 
+var BucketsCanceled = prometheus.NewCounterVec(
+	prometheus.CounterOpts{
+		Name: "cs_bucket_canceled_total",
+		Help: "Total buckets canceled.",
+	},
+	[]string{"name"},
+)
+
 var BucketsUnderflow = prometheus.NewCounterVec(
 	prometheus.CounterOpts{
 		Name: "cs_bucket_underflowed_total",
@@ -153,6 +162,7 @@ func FromFactory(bucketFactory BucketFactory) *Leaky {
 		Queue:           NewQueue(Qsize),
 		CacheSize:       bucketFactory.CacheSize,
 		Out:             make(chan *Queue, 1),
+		Suicide:         make(chan bool, 1),
 		AllOut:          bucketFactory.ret,
 		Capacity:        bucketFactory.Capacity,
 		Leakspeed:       bucketFactory.leakspeed,
@@ -226,7 +236,7 @@ func LeakRoutine(leaky *Leaky) error {
 			if leaky.logger.Level >= log.TraceLevel {
 				leaky.logger.Tracef("Pour event: %s", spew.Sdump(msg))
 			}
-			BucketsPour.With(prometheus.Labels{"name": leaky.Name, "source": msg.Line.Src}).Inc()
+			BucketsPour.With(prometheus.Labels{"name": leaky.Name, "source": msg.Line.Src, "type": msg.Line.Module}).Inc()
 
 			leaky.Pour(leaky, msg) // glue for now
 			//Clear cache on behalf of pour
@@ -237,7 +247,15 @@ func LeakRoutine(leaky *Leaky) error {
 		case ofw := <-leaky.Out:
 			leaky.overflow(ofw)
 			return nil
-			/*we underflow or reach bucket deadline (timers)*/
+		/*suiciiiide*/
+		case <-leaky.Suicide:
+			close(leaky.Signal)
+			BucketsCanceled.With(prometheus.Labels{"name": leaky.Name}).Inc()
+			leaky.logger.Debugf("Suicide triggered")
+			leaky.AllOut <- types.Event{Type: types.OVFLW, Overflow: types.RuntimeAlert{Mapkey: leaky.Mapkey}}
+			leaky.logger.Tracef("Returning from leaky routine.")
+			return nil
+		/*we underflow or reach bucket deadline (timers)*/
 		case <-durationTicker:
 			var (
 				alert types.RuntimeAlert
